From 840dbe71d09a69c94167a9e48935011f071c6c37 Mon Sep 17 00:00:00 2001
From: Kaarlo Raiha <kaarlo.raiha@tut.fi>
Date: Sun, 8 Jun 2008 20:31:16 +0300
Subject: [PATCH] Adds Internet Explorer proxy option for http access module

---
 modules/access/http.c |   96 ++++++++++++++++++++++++++++++++++++++++++++++++-
 1 files changed, 95 insertions(+), 1 deletions(-)

diff --git a/modules/access/http.c b/modules/access/http.c
index 4a1e784..2f63552 100644
--- a/modules/access/http.c
+++ b/modules/access/http.c
@@ -50,6 +50,11 @@
 #   include <zlib.h>
 #endif
 
+/* For Internet Explorer proxy settings */
+#ifdef WIN32
+#include <Windows.h>
+#endif /* WIN32 */
+
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
@@ -60,7 +65,8 @@ static void Close( vlc_object_t * );
 #define PROXY_LONGTEXT N_( \
     "HTTP proxy to be used It must be of the form " \
     "http://[user[:pass]@]myproxy.mydomain:myport/ ; " \
-    "if empty, the http_proxy environment variable will be tried." )
+    "if empty, the http_proxy environment variable will be tried " \
+    "on Unix/Linux operating systems.")
 
 #define CACHING_TEXT N_("Caching value in ms")
 #define CACHING_LONGTEXT N_( \
@@ -76,6 +82,12 @@ static void Close( vlc_object_t * );
     "Automatically try to reconnect to the stream in case of a sudden " \
     "disconnect." )
 
+#ifdef WIN32
+#define IE_PROXY_TEXT N_("Use Internet Explorer proxy settings")
+#define IE_PROXY_LONGTEXT N_( \
+		"Automatically use Internet Explorer proxy settings. Overrides HTTP proxy address!" )
+#endif /* WIN32 */
+
 #define CONTINUOUS_TEXT N_("Continuous stream")
 #define CONTINUOUS_LONGTEXT N_("Read a file that is " \
     "being constantly updated (for example, a JPG file on a server). " \
@@ -94,6 +106,11 @@ vlc_module_begin();
 
     add_string( "http-proxy", NULL, NULL, PROXY_TEXT, PROXY_LONGTEXT,
                 false );
+
+#ifdef WIN32
+    add_bool( "ie-proxy", 0, NULL, IE_PROXY_TEXT, IE_PROXY_LONGTEXT, true );
+#endif /* WIN32 */
+
     add_integer( "http-caching", 4 * DEFAULT_PTS_DELAY / 1000, NULL,
                  CACHING_TEXT, CACHING_LONGTEXT, true );
     add_string( "http-user-agent", COPYRIGHT_MESSAGE , NULL, AGENT_TEXT,
@@ -145,6 +162,11 @@ struct access_sys_t
 
     /* Proxy */
     bool b_proxy;
+
+#ifdef WIN32
+    bool b_ie_proxy;
+#endif /* WIN32 */
+
     vlc_url_t  proxy;
     http_auth_t proxy_auth;
 
@@ -239,6 +261,9 @@ static int OpenWithCookies( vlc_object_t *p_this, vlc_array_t *cookies )
 #endif
     p_sys->fd = -1;
     p_sys->b_proxy = false;
+#ifdef WIN32
+    p_sys->b_ie_proxy = false;
+#endif /* WIN32 */
     p_sys->i_version = 1;
     p_sys->b_seekable = true;
     p_sys->psz_mime = NULL;
@@ -297,13 +322,82 @@ static int OpenWithCookies( vlc_object_t *p_this, vlc_array_t *cookies )
     /* Do user agent */
     p_sys->psz_user_agent = var_CreateGetString( p_access, "http-user-agent" );
 
+    /* Check if we have Internet Explorer proxy enabled in GUI */
+#ifdef WIN32
+    p_sys->b_ie_proxy = var_CreateGetBool( p_access, "ie-proxy" );
+#endif /* WIN32 */
+
     /* Check proxy */
+#ifdef WIN32
+    psz = var_CreateGetString( p_access, "http-proxy" );
+
+    /* We either have proxy URL or IE proxy setting enabled */
+    if( *psz || p_sys->b_ie_proxy )
+    {
+        p_sys->b_proxy = true;
+        bool b_proxyenabled = false;
+        if( p_sys->b_ie_proxy )
+        { /* Read IE proxy settings from Windows registry */
+            HKEY hkey = 0;
+            if( ERROR_SUCCESS == RegOpenKeyEx( HKEY_CURRENT_USER, "Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings", 0, KEY_READ, &hkey ) )
+            {
+                DWORD bval = 0;
+                DWORD bvalsize = sizeof( bval );
+                DWORD t = 0;
+                long ret = RegQueryValueEx( hkey, _T("ProxyEnable"), NULL, &t, (LPBYTE) &bval, &bvalsize );
+                if( ERROR_SUCCESS == ret)
+                {
+                    if( bval != 0 ) /* check that ProxyEnable is 1 and not 0 */
+                    {
+                        char szVal[256];
+                        DWORD dwSize = sizeof( szVal );
+                        b_proxyenabled = true;
+                        if( ERROR_SUCCESS == RegQueryValueEx( hkey, _T("ProxyServer"), NULL, NULL, (BYTE*)szVal, &dwSize ) )
+                        {
+                            free( psz );
+                            psz = ( char * ) malloc( dwSize + 1 );
+                            if( !psz ) return VLC_ENOMEM;
+                            strncpy( psz, szVal, dwSize );
+                            psz[dwSize] = 0;
+                            msg_Dbg( p_access, "IE proxy is: %s", psz );
+                        }
+                        else
+                        {
+                            msg_Warn( p_access, "Cannot open ProxyServer registry key" );
+                        }
+                    }
+                    else
+                    {
+                        msg_Warn( p_access, "ProxyEnable registry key is 0" );
+                    }
+                }
+                else
+                {
+                    msg_Warn( p_access, "Cannot open ProxyEnable registry key" );
+                    msg_Warn( p_access, "Error code: %lu", ret );
+                }
+                RegCloseKey( hkey );
+            }
+            else
+            {
+                msg_Warn( p_access, "Cannot open Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings" );
+            }
+        }
+        /* We either have proxy URL or we got IE proxy settings */
+        if(*psz || b_proxyenabled )
+        {
+             vlc_UrlParse( &p_sys->proxy, psz, 0 );
+        }
+    }
+#else
     psz = var_CreateGetString( p_access, "http-proxy" );
     if( *psz )
     {
         p_sys->b_proxy = true;
         vlc_UrlParse( &p_sys->proxy, psz, 0 );
     }
+#endif /* WIN32 */
+
 #ifdef HAVE_GETENV
     else
     {
-- 
1.5.5.1

